/*
FILENAME...   NanotecDriver.h
USAGE...      Motor driver support for the Nanotec SMCI controller.

Mark Koennecke
July 2015

*/

#include "asynMotorController.h"
#include "asynMotorAxis.h"

#define COMLEN 80
#define MAXMOT 99

class NanotecAxis : public asynMotorAxis
{
public:
  /* These are the methods we override from the base class */
  NanotecAxis(class NanotecController *pC, int axis, int busAddress);
  void report(FILE *fp, int level);
  asynStatus move(double position, int relative, double min_velocity, double max_velocity, double acceleration);
  asynStatus moveVelocity(double min_velocity, double max_velocity, double acceleration);
  asynStatus home(double min_velocity, double max_velocity, double acceleration, int forwards);
  asynStatus stop(double acceleration);
  asynStatus poll(bool *moving);
  asynStatus setPosition(double position);
  asynStatus setClosedLoop(bool closedLoop);

private:
  NanotecController *pC_;          /**< Pointer to the asynMotorController to which this axis belongs.
                                   *   Abbreviated because it is used very frequently */
  double position;
  int homing;
  time_t next_poll;
  int busAddress;
  
friend class NanotecController;
};

class NanotecController : public asynMotorController {
public:
  NanotecController(const char *portName, const char *NanotecPortName, int numMot, const char *busAddresses);

  void report(FILE *fp, int level);
  NanotecAxis* getAxis(asynUser *pasynUser);
  NanotecAxis* getAxis(int axisNo);

friend class NanotecAxis;
 private:
 asynUser *pasynUserController_;

 asynStatus transactController(char command[COMLEN], char reply[COMLEN]);



};
